@@ -1,4 +1,4 @@
-# Copyright 2015, 2016 IBM Corp.
+# Copyright 2015, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -21,11 +21,10 @@ from nova_powervm import conf as cfg
 from nova_powervm.virt.powervm.disk import driver as disk_drv
 from nova_powervm.virt.powervm import exception as npvmex
 from nova_powervm.virt.powervm.i18n import _
-from nova_powervm.virt.powervm.i18n import _LE
-from nova_powervm.virt.powervm.i18n import _LI
 from nova_powervm.virt.powervm import vm
 
 from nova import image
+from oslo_utils import excutils
 import pypowervm.const as pvm_const
 from pypowervm.tasks import cluster_ssp as tsk_cs
 from pypowervm.tasks import partition as tsk_par
@@ -47,9 +46,8 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
     Shared Storage Pools are a clustered file system technology that can link
     together Virtual I/O Servers.
 
-    This adapter provides the connection for nova based storage (not Cinder)
-    to connect to virtual machines.  A separate Cinder driver for SSPs may
-    exist in the future.
+    This adapter provides the connection for nova ephemeral storage (not
+    Cinder) to connect to virtual machines.
     """
 
     capabilities = {
@@ -59,7 +57,8 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
     def __init__(self, adapter, host_uuid):
         """Initialize the SSPDiskAdapter.
 
-        :param connection: connection information for the underlying driver
+        :param adapter: pypowervm.adapter.Adapter for the PowerVM REST API.
+        :param host_uuid: PowerVM UUID of the managed system.
         """
         super(SSPDiskAdapter, self).__init__(adapter, host_uuid)
 
@@ -70,9 +69,8 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
         self.ssp_name = self._ssp.name
         self.tier_name = self._tier.name
 
-        LOG.info(_LI("SSP Storage driver initialized. "
-                     "Cluster '%(clust_name)s'; SSP '%(ssp_name)s'; "
-                     "Tier '%(tier_name)s"),
+        LOG.info("SSP Storage driver initialized. Cluster '%(clust_name)s'; "
+                 "SSP '%(ssp_name)s'; Tier '%(tier_name)s",
                  {'clust_name': self.clust_name, 'ssp_name': self.ssp_name,
                   'tier_name': self.tier_name})
 
@@ -117,11 +115,9 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
                      'source_ssp_name': disk_info.get('ssp_name')}
                     )
 
-    def disconnect_image_disk(self, context, instance, stg_ftsk=None,
-                              disk_type=None):
+    def disconnect_disk(self, instance, stg_ftsk=None, disk_type=None):
         """Disconnects the storage adapters from the image disk.
 
-        :param context: nova context for operation
         :param instance: instance to disconnect the image for.
         :param stg_ftsk: (Optional) The pypowervm transaction FeedTask for
                          the I/O Operations.  If provided, the Virtual I/O
@@ -143,9 +139,8 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
 
         # Delay run function to remove the mapping between the VM and the LU
         def rm_func(vios_w):
-            LOG.info(_LI("Removing SSP disk connection between VM %(vm)s and "
-                         "VIOS %(vios)s."),
-                     {'vm': instance.name, 'vios': vios_w.name})
+            LOG.info("Removing SSP disk connection to VIOS %(vios)s.",
+                     {'vios': vios_w.name}, instance=instance)
             return tsk_map.remove_maps(vios_w, lpar_uuid,
                                        match_func=match_func)
 
@@ -187,27 +182,23 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
         """
         tsk_map.remove_lu_mapping(self.adapter, vios_uuid, self.mp_uuid,
                                   disk_names=[disk_name])
-        LOG.info(_LI(
-            "Unmapped boot disk %(disk_name)s from the management partition "
-            "from Virtual I/O Server %(vios_uuid)s."), {
-                'disk_name': disk_name, 'mp_uuid': self.mp_uuid,
-                'vios_uuid': vios_uuid})
+        LOG.info("Unmapped boot disk %(disk_name)s from the management "
+                 "partition from Virtual I/O Server %(vios_uuid)s.",
+                 {'disk_name': disk_name, 'mp_uuid': self.mp_uuid,
+                  'vios_uuid': vios_uuid})
 
-    def delete_disks(self, context, instance, storage_elems):
+    def delete_disks(self, storage_elems):
         """Removes the disks specified by the mappings.
 
-        :param context: nova context for operation
-        :param instance: instance to delete the disk for.
         :param storage_elems: A list of the storage elements (LU
                               ElementWrappers) that are to be deleted.  Derived
-                              from the return value from disconnect_image_disk.
+                              from the return value from disconnect_disk.
         """
         tsk_stg.rm_tier_storage(storage_elems, tier=self._tier)
 
     def _create_disk_from_image(self, context, instance, image_meta,
-                                disk_size_gb,
                                 image_type=disk_drv.DiskType.BOOT):
-        """Creates a boot disk and links the specified image to it.
+        """Creates a disk and copies the specified image to it.
 
         If the specified image has not already been uploaded, an Image LU is
         created for it.  A Disk LU is then created for the instance and linked
@@ -217,31 +208,25 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
         :param instance: instance to create the disk for.
         :param nova.objects.ImageMeta image_meta:
             The metadata of the image of the instance.
-        :param disk_size_gb: The size of the disk to create in GB.  If smaller
-                             than the image, it will be ignored (as the disk
-                             must be at least as big as the image).  Must be an
-                             int.
         :param image_type: The image type. See disk_drv.DiskType.
         :return: The backing pypowervm LU storage object that was created.
         """
-        LOG.info(_LI('SSP: Create %(image_type)s disk from image %(image_id)s '
-                     'for instance %(instance_uuid)s.'),
-                 dict(image_type=image_type, image_id=image_meta.id,
-                      instance_uuid=instance.uuid))
-
-        def upload(path):
-            IMAGE_API.download(context, image_meta.id, dest_path=path)
+        LOG.info('SSP: Create %(image_type)s disk from image %(image_id)s.',
+                 dict(image_type=image_type, image_id=image_meta.id),
+                 instance=instance)
 
         image_lu = tsk_cs.get_or_upload_image_lu(
             self._tier, self._get_image_name(image_meta),
-            self._any_vios_uuid(), upload,
-            image_meta.size, upload_type=tsk_stg.UploadType.FUNC)
+            self._any_vios_uuid(), disk_drv.IterableToFileAdapter(
+                IMAGE_API.download(context, image_meta.id)),
+            image_meta.size, upload_type=tsk_stg.UploadType.IO_STREAM)
 
         boot_lu_name = self._get_disk_name(image_type, instance)
-        LOG.info(_LI('SSP: Disk name is %s'), boot_lu_name)
+        LOG.info('SSP: Disk name is %s', boot_lu_name, instance=instance)
 
-        return tsk_stg.crt_lu(self._tier, boot_lu_name, disk_size_gb,
-                              typ=pvm_stg.LUType.DISK, clone=image_lu)[1]
+        return tsk_stg.crt_lu(
+            self._tier, boot_lu_name, instance.flavor.root_gb,
+            typ=pvm_stg.LUType.DISK, clone=image_lu)[1]
 
     def get_disk_ref(self, instance, disk_type):
         """Returns a reference to the disk for the instance."""
@@ -250,10 +235,9 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
             self.adapter, parent=self._tier, name=lu_name,
             lu_type=pvm_stg.LUType.DISK, one_result=True)
 
-    def connect_disk(self, context, instance, disk_info, stg_ftsk=None):
+    def connect_disk(self, instance, disk_info, stg_ftsk=None):
         """Connects the disk image to the Virtual Machine.
 
-        :param context: nova context for the transaction.
         :param instance: nova instance to connect the disk to.
         :param disk_info: The pypowervm storage element returned from
                           create_disk_from_image.  Ex. VOptMedia, VDisk, LU,
@@ -275,9 +259,8 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
 
         # This is the delay apply mapping
         def add_func(vios_w):
-            LOG.info(_LI("Adding SSP disk connection between VM %(vm)s and "
-                         "VIOS %(vios)s."),
-                     {'vm': instance.name, 'vios': vios_w.name})
+            LOG.info("Adding SSP disk connection to VIOS %(vios)s.",
+                     {'vios': vios_w.name}, instance=instance)
             mapping = tsk_map.build_vscsi_mapping(
                 self.host_uuid, vios_w, lpar_uuid, lu)
             return tsk_map.add_map(vios_w, mapping)
@@ -295,16 +278,6 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
         if stg_ftsk.name == 'ssp':
             stg_ftsk.execute()
 
-    def extend_disk(self, context, instance, disk_info, size):
-        """Extends the disk.
-
-        :param context: nova context for operation.
-        :param instance: instance to extend the disk for.
-        :param disk_info: dictionary with disk info.
-        :param size: the new size in gb.
-        """
-        raise NotImplementedError()
-
     def check_instance_shared_storage_local(self, context, instance):
         """Check if instance files located on shared storage.
 
@@ -331,9 +304,8 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
                 ssp_uuid = data.get('ssp_uuid')
                 if ssp_uuid is not None:
                     return ssp_uuid == self._cluster.ssp_uuid
-        except Exception as e:
-            LOG.exception(_LE(u'Error checking for shared storage. '
-                              'exception=%s'), e)
+        except Exception:
+            LOG.exception('Error checking for shared storage.')
         return False
 
     def check_instance_shared_storage_cleanup(self, context, data):
@@ -376,17 +348,16 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
             else:
                 # Otherwise, pull the entire feed of Clusters and, if
                 # exactly one result, use it.
-                resp = self.adapter.read(pvm_clust.Cluster.schema_type)
-                wraps = pvm_clust.Cluster.wrap(resp)
+                wraps = pvm_clust.Cluster.get(self.adapter)
                 if len(wraps) == 0:
                     raise npvmex.NoConfigNoClusterFound()
                 if len(wraps) > 1:
                     raise npvmex.NoConfigTooManyClusters(
                         clust_count=len(wraps))
             clust_wrap = wraps[0]
-        except Exception as e:
-            LOG.exception(e.message)
-            raise e
+        except Exception:
+            with excutils.save_and_reraise_exception(logger=LOG):
+                LOG.exception("PowerVM error fetching cluster.")
         return clust_wrap
 
     def _refresh_cluster(self):
@@ -415,8 +386,8 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
         """
         # TODO(IBM): Smarter refreshing (i.e. don't do it every time).
         if getattr(self, '_ssp_wrap', None) is None:
-            resp = self.adapter.read_by_href(self._cluster.ssp_uri)
-            self._ssp_wrap = pvm_stg.SSP.wrap(resp)
+            self._ssp_wrap = pvm_stg.SSP.get_by_href(self.adapter,
+                                                     self._cluster.ssp_uri)
         else:
             self._ssp_wrap = self._ssp_wrap.refresh()
         return self._ssp_wrap
