@@ -112,7 +112,7 @@ class VolumePreMigrationFailed(nex.NovaException):
 
 
 class PowerVMAPIFailed(nex.NovaException):
-    msg_fmt = _("PowerVM API Failed to complete for instance=%(inst_name)s."
+    msg_fmt = _("PowerVM API failed to complete for instance=%(inst_name)s."
                 "%(reason)s")
 
 
@@ -125,6 +125,15 @@ class ViosNotAvailable(nex.NovaException):
                 "I/O Servers and then restart the Nova Compute Agent.")
 
 
+class NoActiveViosForFeedTask(nex.NovaException):
+    msg_fmt = _("There are no active Virtual I/O Servers available.")
+
+
 class InvalidRebuild(nex.NovaException):
     msg_fmt = _("Unable to rebuild virtual machine on new host.  Error is "
                 "%(error)s")
+
+
+class OptRequiredIfOtherOptValue(nex.NovaException):
+    msg_fmt = _("The %(then_opt)s option is required if %(if_opt)s is "
+                "specified as '%(if_value)s'.")
