@@ -26,6 +26,7 @@ from nova import utils
 from oslo_config import cfg
 from oslo_log import log
 from oslo_serialization import jsonutils
+from oslo_utils import excutils
 from oslo_utils import importutils
 from pypowervm import exceptions as pvm_ex
 from pypowervm.tasks import cna as pvm_cna
@@ -39,11 +40,9 @@ from pypowervm.wrappers import managed_system as pvm_ms
 from pypowervm.wrappers import network as pvm_net
 
 from nova_powervm.virt.powervm.i18n import _
-from nova_powervm.virt.powervm.i18n import _LE
-from nova_powervm.virt.powervm.i18n import _LI
-from nova_powervm.virt.powervm.i18n import _LW
 from nova_powervm.virt.powervm import vm
 
+
 LOG = log.getLogger(__name__)
 
 SECURE_RMC_VSWITCH = 'MGMTSWITCH'
@@ -113,12 +112,12 @@ def _push_vif_event(adapter, action, vif_w, instance, vif_type):
     event = pvm_evt.Event.bld(adapter, data, detail)
     try:
         event = event.create()
-        LOG.debug(_LI('Pushed custom event for consumption by neutron agent: '
-                      '%s'), str(event), instance=instance)
+        LOG.debug('Pushed custom event for consumption by neutron agent: %s',
+                  str(event), instance=instance)
     except Exception:
-        LOG.error(_LE('Custom VIF event push failed.  %s'), str(event),
-                  instance=instance)
-        raise
+        with excutils.save_and_reraise_exception(logger=LOG):
+            LOG.exception('Custom VIF event push failed.  %s', str(event),
+                          instance=instance)
 
 
 def plug(adapter, host_uuid, instance, vif, slot_mgr, new_vif=True):
@@ -148,7 +147,8 @@ def plug(adapter, host_uuid, instance, vif, slot_mgr, new_vif=True):
         vnet_w = vif_drv.plug(vif, slot_num, new_vif=new_vif)
     except pvm_ex.HttpError as he:
         # Log the message constructed by HttpError
-        LOG.exception(he.args[0], instance=instance)
+        LOG.exception("HttpError during vif plug operation.",
+                      instance=instance)
         raise exception.VirtualInterfacePlugException(reason=he.args[0])
     # Other exceptions are (hopefully) custom VirtualInterfacePlugException
     # generated lower in the call stack.
@@ -186,7 +186,8 @@ def unplug(adapter, host_uuid, instance, vif, slot_mgr, cna_w_list=None):
             _push_vif_event(adapter, 'unplug', vnet_w, instance, vif['type'])
     except pvm_ex.HttpError as he:
         # Log the message constructed by HttpError
-        LOG.exception(he.args[0], instance=instance)
+        LOG.exception("HttpError during vif unplug operation.",
+                      instance=instance)
         raise exception.VirtualInterfaceUnplugException(reason=he.args[0])
 
     if vnet_w:
@@ -368,24 +369,18 @@ class PvmVifDriver(object):
 
         cna_w = self._find_cna_for_vif(cna_w_list, vif)
         if not cna_w:
-            LOG.warning(_LW('Unable to unplug VIF with mac %(mac)s for '
-                            'instance %(inst)s.  The VIF was not found on '
-                            'the instance.'),
-                        {'mac': vif['address'], 'inst': self.instance.name},
-                        instance=self.instance)
+            LOG.warning('Unable to unplug VIF with mac %(mac)s.  The VIF was '
+                        'not found on the instance.',
+                        {'mac': vif['address']}, instance=self.instance)
             return None
 
-        LOG.info(_LI('Deleting VIF with mac %(mac)s for instance %(inst)s.'),
-                 {'mac': vif['address'], 'inst': self.instance.name},
-                 instance=self.instance)
+        LOG.info('Deleting VIF with mac %(mac)s.',
+                 {'mac': vif['address']}, instance=self.instance)
         try:
             cna_w.delete()
         except Exception as e:
-            LOG.error(_LE('Unable to unplug VIF with mac %(mac)s for instance '
-                          '%(inst)s.'),
-                      {'mac': vif['address'], 'inst': self.instance.name},
-                      instance=self.instance)
-            LOG.exception(e, instance=self.instance)
+            LOG.exception('Unable to unplug VIF with mac %(mac)s.',
+                          {'mac': vif['address']}, instance=self.instance)
             raise exception.VirtualInterfaceUnplugException(
                 reason=six.text_type(e))
         return cna_w
@@ -484,7 +479,8 @@ class PvmSeaVifDriver(PvmVifDriver):
         if not vlan:
             vlan = int(vif['details']['vlan'])
 
-        LOG.debug("Creating SEA based VIF with VLAN %s", str(vlan))
+        LOG.debug("Creating SEA-based VIF with VLAN %s", str(vlan),
+                  instance=self.instance)
         cna_w = pvm_cna.crt_cna(self.adapter, self.host_uuid, lpar_uuid, vlan,
                                 mac_addr=vif['address'], slot_num=slot_num)
 
@@ -599,11 +595,9 @@ class PvmLBVifDriver(PvmLioVifDriver):
         # Find the CNA for this vif.
         cna_w = self._find_cna_for_vif(cna_w_list, vif)
         if not cna_w:
-            LOG.warning(_LW('Unable to unplug VIF with mac %(mac)s for '
-                            'instance %(inst)s.  The VIF was not found on '
-                            'the instance.'),
-                        {'mac': vif['address'], 'inst': self.instance.name},
-                        instance=self.instance)
+            LOG.warning('Unable to unplug VIF with mac %(mac)s.  The VIF was '
+                        'not found on the instance.',
+                        {'mac': vif['address']}, instance=self.instance)
             return None
 
         # Find and delete the trunk adapters
@@ -614,12 +608,11 @@ class PvmLBVifDriver(PvmLioVifDriver):
         try:
             utils.execute('brctl', 'delif', vif['network']['bridge'],
                           dev_name, run_as_root=True)
-        except Exception as e:
-            LOG.warning(_LW('Unable to delete device %(dev_name)s from bridge '
-                            '%(bridge)s. Error: %(error)s'),
-                        {'dev_name': dev_name,
-                         'bridge': vif['network']['bridge'],
-                         'error': e.message}, instance=self.instance)
+        except Exception:
+            LOG.exception(
+                'Unable to delete device %(dev_name)s from bridge %(bridge)s.',
+                {'dev_name': dev_name, 'bridge': vif['network']['bridge']},
+                instance=self.instance)
         for trunk in trunks:
             trunk.delete()
 
@@ -647,10 +640,8 @@ class PvmVnicSriovVifDriver(PvmVifDriver):
             network = napi.get(admin_context, net_id)
             physnet = network.physical_network
 
-        LOG.debug("Plugging vNIC SR-IOV vif for physical network "
-                  "'%(physnet)s' into instance %(inst)s.",
-                  {'physnet': physnet, 'inst': self.instance.name},
-                  instance=self.instance)
+        LOG.debug("Plugging vNIC SR-IOV vif for physical network %(physnet)s.",
+                  {'physnet': physnet}, instance=self.instance)
 
         # Get the msys
         msys = pvm_ms.System.get(self.adapter)[0]
@@ -696,11 +687,9 @@ class PvmVnicSriovVifDriver(PvmVifDriver):
         vnic = vm.get_vnics(
             self.adapter, self.instance, mac=mac, one_result=True)
         if not vnic:
-            LOG.warning(_LW('Unable to unplug VIF with mac %(mac)s for '
-                            'instance %(inst)s.  No matching vNIC was found '
-                            'on the instance.  VIF: %(vif)s'),
-                        {'mac': mac, 'inst': self.instance.name, 'vif': vif},
-                        instance=self.instance)
+            LOG.warning('Unable to unplug VIF with mac %(mac)s. No matching '
+                        'vNIC was found on the instance. VIF: %(vif)s',
+                        {'mac': mac, 'vif': vif}, instance=self.instance)
             return None
         vnic.delete()
         return vnic
@@ -769,11 +758,9 @@ class PvmOvsVifDriver(PvmLioVifDriver):
         # Find the CNA for this vif.
         cna_w = self._find_cna_for_vif(cna_w_list, vif)
         if not cna_w:
-            LOG.warning(_LW('Unable to unplug VIF with mac %(mac)s for '
-                            'instance %(inst)s.  The VIF was not found on '
-                            'the instance.'),
-                        {'mac': vif['address'], 'inst': self.instance.name},
-                        instance=self.instance)
+            LOG.warning('Unable to unplug VIF with mac %(mac)s for. The VIF '
+                        'was not found on the instance.',
+                        {'mac': vif['address']}, instance=self.instance)
             return None
 
         # Find and delete the trunk adapters
@@ -801,6 +788,8 @@ class PvmOvsVifDriver(PvmLioVifDriver):
                                   mac address, value is the destination's
                                   target hypervisor VLAN.
         """
+        self._cleanup_orphan_adapters(vif,
+                                      CONF.powervm.pvm_vswitch_for_novalink_io)
         mgmt_wrap = pvm_par.get_this_partition(self.adapter)
         dev = self.get_trunk_dev_name(vif)
 
@@ -819,8 +808,8 @@ class PvmOvsVifDriver(PvmLioVifDriver):
 
         # Save this data for the migration command.
         vea_vlan_mappings[vif['address']] = cna_w.pvid
-        LOG.info(_LI("VIF with mac %(mac)s is going on trunk %(dev)s with "
-                     "PVID %(pvid)s"),
+        LOG.info("VIF with mac %(mac)s is going on trunk %(dev)s with PVID "
+                 "%(pvid)s",
                  {'mac': vif['address'], 'dev': dev, 'pvid': cna_w.pvid},
                  instance=self.instance)
 
@@ -838,9 +827,8 @@ class PvmOvsVifDriver(PvmLioVifDriver):
                                   mac address, value is the destination's
                                   target hypervisor VLAN.
         """
-        LOG.warning(_LW("Rolling back the live migrate of VIF with mac "
-                        "%(mac)s."), {'mac': vif['address']},
-                    instance=self.instance)
+        LOG.warning("Rolling back the live migrate of VIF with mac %(mac)s.",
+                    {'mac': vif['address']}, instance=self.instance)
 
         # We know that we just attached the VIF to the NovaLink VM.  Search
         # for a trunk adapter with the PVID and vSwitch that we specified
@@ -868,8 +856,8 @@ class PvmOvsVifDriver(PvmLioVifDriver):
 
         if trunk:
             # Delete the peer'd trunk adapter.
-            LOG.warning(_LW("Deleting target side trunk adapter %(dev)s for "
-                            "rollback operation"), {'dev': trunk.dev_name},
+            LOG.warning("Deleting target side trunk adapter %(dev)s for "
+                        "rollback operation", {'dev': trunk.dev_name},
                         instance=self.instance)
             trunk.delete()
 
@@ -903,3 +891,13 @@ class PvmOvsVifDriver(PvmLioVifDriver):
         """
         linux_net.delete_ovs_vif_port(vif['network']['bridge'],
                                       self.get_trunk_dev_name(vif))
+
+    def _cleanup_orphan_adapters(self, vif, vswitch_name):
+        """Finds and removes trunk VEAs that have no corresponding CNA."""
+        # Find and delete orphan adapters with macs matching our vif
+        orphans = pvm_cna.find_orphaned_trunks(self.adapter, vswitch_name)
+        for orphan in orphans:
+            if vm.norm_mac(orphan.mac) == vif['address']:
+                linux_net.delete_ovs_vif_port(vif['network']['bridge'],
+                                              self.get_trunk_dev_name(vif))
+                orphan.delete()
