@@ -1,4 +1,4 @@
-# Copyright 2015, 2016 IBM Corp.
+# Copyright 2015, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -19,9 +19,6 @@ from oslo_log import log as logging
 
 from nova_powervm import conf as cfg
 from nova_powervm.virt.powervm import exception as p_exc
-from nova_powervm.virt.powervm.i18n import _LE
-from nova_powervm.virt.powervm.i18n import _LI
-from nova_powervm.virt.powervm.i18n import _LW
 from nova_powervm.virt.powervm import vm
 from nova_powervm.virt.powervm.volume import driver as v_driver
 from nova_powervm.virt.powervm.volume import volume as volume
@@ -33,6 +30,8 @@ from pypowervm.utils import transaction as tx
 from pypowervm.wrappers import virtual_io_server as pvm_vios
 
 import six
+
+
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
@@ -107,9 +106,8 @@ class PVVscsiFCVolumeAdapter(volume.VscsiVolumeAdapter,
 
         # See the connect_volume for why this is a direct call instead of
         # using the tx_mgr.feed
-        vios_feed = self.adapter.read(pvm_vios.VIOS.schema_type,
-                                      xag=[pvm_const.XAG.VIO_STOR])
-        vios_wraps = pvm_vios.VIOS.wrap(vios_feed)
+        vios_wraps = pvm_vios.VIOS.get(self.adapter,
+                                       xag=[pvm_const.XAG.VIO_STOR])
 
         # Iterate through host vios list to find valid hdisks.
         for vios_w in vios_wraps:
@@ -181,23 +179,26 @@ class PVVscsiFCVolumeAdapter(volume.VscsiVolumeAdapter,
         # Server (if any).
         itls = hdisk.build_itls(vio_wwpns, t_wwpns, lun)
         if len(itls) == 0:
-            LOG.debug('No ITLs for VIOS %(vios)s for volume %(volume_id)s.'
-                      % {'vios': vios_w.name, 'volume_id': volume_id})
+            LOG.debug('No ITLs for VIOS %(vios)s for volume %(volume_id)s.',
+                      {'vios': vios_w.name, 'volume_id': volume_id},
+                      instance=self.instance)
             return None, None, None
 
         status, device_name, udid = hdisk.discover_hdisk(self.adapter,
                                                          vios_w.uuid, itls)
 
         if hdisk.good_discovery(status, device_name):
-            LOG.info(_LI('Discovered %(hdisk)s on vios %(vios)s for '
-                     'volume %(volume_id)s. Status code: %(status)s.'),
+            LOG.info('Discovered %(hdisk)s on vios %(vios)s for volume '
+                     '%(volume_id)s. Status code: %(status)s.',
                      {'hdisk': device_name, 'vios': vios_w.name,
-                      'volume_id': volume_id, 'status': str(status)})
+                      'volume_id': volume_id, 'status': str(status)},
+                     instance=self.instance)
         elif status == hdisk.LUAStatus.DEVICE_IN_USE:
-            LOG.warning(_LW('Discovered device %(dev)s for volume %(volume)s '
-                            'on %(vios)s is in use. Error code: %(status)s.'),
+            LOG.warning('Discovered device %(dev)s for volume %(volume)s '
+                        'on %(vios)s is in use. Error code: %(status)s.',
                         {'dev': device_name, 'volume': volume_id,
-                         'vios': vios_w.name, 'status': str(status)})
+                         'vios': vios_w.name, 'status': str(status)},
+                        instance=self.instance)
 
         return status, device_name, udid
 
@@ -219,9 +220,10 @@ class PVVscsiFCVolumeAdapter(volume.VscsiVolumeAdapter,
         slot, lua = slot_mgr.build_map.get_vscsi_slot(vios_w, udid)
 
         if slot_mgr.is_rebuild and not slot:
-            LOG.debug('Detected a device with UDID %s on VIOS %s on the '
-                      'rebuild that did not exist on the source.  '
-                      'Ignoring.', udid, vios_w.uuid)
+            LOG.debug('Detected a device with UDID %(udid)s on VIOS '
+                      '%(vios)s on the rebuild that did not exist on the '
+                      'source. Ignoring.', {'udid': udid, 'vios': vios_w.uuid},
+                      instance=self.instance)
             return False
 
         if hdisk.good_discovery(status, device_name):
@@ -234,7 +236,8 @@ class PVVscsiFCVolumeAdapter(volume.VscsiVolumeAdapter,
             # Save the UDID for the disk in the connection info.  It is
             # used for the detach.
             self._set_udid(udid)
-            LOG.debug('Device attached: %s', device_name)
+            LOG.debug('Added deferred task to attach device %s', device_name,
+                      instance=self.instance)
 
             # Valid attachment
             return True
@@ -255,7 +258,8 @@ class PVVscsiFCVolumeAdapter(volume.VscsiVolumeAdapter,
                      otherwise.
             """
             LOG.debug("Disconnect volume %(vol)s from vios uuid %(uuid)s",
-                      dict(vol=self.volume_id, uuid=vios_w.uuid))
+                      dict(vol=self.volume_id, uuid=vios_w.uuid),
+                      instance=self.instance)
             udid, device_name = None, None
             try:
                 udid = self._get_udid()
@@ -274,29 +278,31 @@ class PVVscsiFCVolumeAdapter(volume.VscsiVolumeAdapter,
                     # in the I/O Server.  Subsequent scrub code on future
                     # deploys will clean this up.
                     if not hdisk.good_discovery(status, device_name):
-                        LOG.warning(_LW(
+                        LOG.warning(
                             "Disconnect Volume: The backing hdisk for volume "
                             "%(volume_id)s on Virtual I/O Server %(vios)s is "
                             "not in a valid state.  This may be the result of "
-                            "an evacuate."),
-                            {'volume_id': self.volume_id, 'vios': vios_w.name})
+                            "an evacuate.",
+                            {'volume_id': self.volume_id, 'vios': vios_w.name},
+                            instance=self.instance)
                         return False
 
-            except Exception as e:
-                LOG.warning(_LW(
+            except Exception:
+                LOG.exception(
                     "Disconnect Volume: Failed to find disk on Virtual I/O "
                     "Server %(vios_name)s for volume %(volume_id)s. Volume "
-                    "UDID: %(volume_uid)s.  Error: %(error)s"),
-                    {'error': e, 'volume_uid': udid, 'vios_name': vios_w.name,
-                     'volume_id': self.volume_id})
+                    "UDID: %(volume_uid)s.",
+                    {'volume_uid': udid, 'vios_name': vios_w.name,
+                     'volume_id': self.volume_id}, instance=self.instance)
                 return False
 
             # We have found the device name
-            LOG.info(_LI("Disconnect Volume: Discovered the device %(hdisk)s "
-                         "on Virtual I/O Server %(vios_name)s for volume "
-                         "%(volume_id)s.  Volume UDID: %(volume_uid)s."),
+            LOG.info("Disconnect Volume: Discovered the device %(hdisk)s "
+                     "on Virtual I/O Server %(vios_name)s for volume "
+                     "%(volume_id)s.  Volume UDID: %(volume_uid)s.",
                      {'volume_uid': udid, 'volume_id': self.volume_id,
-                      'vios_name': vios_w.name, 'hdisk': device_name})
+                      'vios_name': vios_w.name, 'hdisk': device_name},
+                     instance=self.instance)
 
             # Add the action to remove the mapping when the stg_ftsk is run.
             partition_id = vm.get_vm_id(self.adapter, self.vm_uuid)
@@ -324,16 +330,14 @@ class PVVscsiFCVolumeAdapter(volume.VscsiVolumeAdapter,
             # Warn if no hdisks disconnected.
             if not any([result['vio_modified']
                         for result in ret['wrapper_task_rets'].values()]):
-                LOG.warning(_LW("Disconnect Volume: Failed to disconnect the "
-                                "volume %(volume_id)s on ANY of the Virtual "
-                                "I/O Servers for instance %(inst)s."),
-                            {'inst': self.instance.name,
-                             'volume_id': self.volume_id})
+                LOG.warning("Disconnect Volume: Failed to disconnect the "
+                            "volume %(volume_id)s on ANY of the Virtual "
+                            "I/O Servers.", {'volume_id': self.volume_id},
+                            instance=self.instance)
 
         except Exception as e:
-            LOG.error(_LE('Cannot detach volumes from virtual machine: %s'),
-                      self.vm_uuid)
-            LOG.exception(_LE('Error: %s'), e)
+            LOG.exception('PowerVM error detaching volume from virtual '
+                          'machine.', instance=self.instance)
             ex_args = {'volume_id': self.volume_id, 'reason': six.text_type(e),
                        'instance_name': self.instance.name}
             raise p_exc.VolumeDetachFailed(**ex_args)
@@ -350,13 +354,6 @@ class PVVscsiFCVolumeAdapter(volume.VscsiVolumeAdapter,
             _vscsi_pfc_wwpns = pvm_tpar.get_physical_wwpns(self.adapter)
         return _vscsi_pfc_wwpns
 
-    def host_name(self):
-        """Derives the host name that should be used for the storage device.
-
-        :return: The host name.
-        """
-        return CONF.host
-
     def _get_hdisk_itls(self, vios_w):
         """Returns the mapped ITLs for the hdisk for the given VIOS.
 
